<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2011 QMT Productions
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_5/index.php $
 * $Id: index.php 1848 2011-04-02 00:12:46Z Quietust $
 */

date_default_timezone_set('UTC');
ignore_user_abort(1);
set_time_limit(0);

// don't allow CLI invocation
if (!isset($_SERVER['REQUEST_URI']))
	die("Access denied");

// prevent recursion
if (defined('IN_GAME'))
	die("Access denied");

define('IN_GAME', TRUE);
define('PROM_BASEDIR', dirname(__FILE__) . '/');

// Don't allow accessing anything while setup.php is being run
if (file_exists(PROM_BASEDIR .'setup.php'))
	die("Access denied");

require_once(PROM_BASEDIR .'config.php');
require_once(PROM_BASEDIR .'includes/constants.php');
require_once(PROM_BASEDIR .'includes/database.php');
require_once(PROM_BASEDIR .'includes/language.php');
require_once(PROM_BASEDIR .'includes/html.php');
require_once(PROM_BASEDIR .'includes/logging.php');
require_once(PROM_BASEDIR .'includes/misc.php');
require_once(PROM_BASEDIR .'includes/news.php');
require_once(PROM_BASEDIR .'includes/permissions.php');
require_once(PROM_BASEDIR .'classes/prom_vars.php');
require_once(PROM_BASEDIR .'classes/prom_user.php');
require_once(PROM_BASEDIR .'classes/prom_empire.php');
require_once(PROM_BASEDIR .'classes/prom_clan.php');
require_once(PROM_BASEDIR .'classes/prom_session.php');
require_once(PROM_BASEDIR .'classes/prom_turns.php');
require_once(PROM_BASEDIR .'includes/auth.php');

function validate_location ($page)
{
	// pages that work from anywhere
	if (in_array($page, array('topplayers', 'topempires', 'topclans', 'playerstats', 'history', 'login', 'signup', 'count', 'pguide', 'credits', 'admin/locks')))
		return $page;
	// special case - go to "login" page when you still have an active session
	elseif ($page == 'relogin')
		return 'main';
	// error trap - all pages beyond this point require some sort of referer
	elseif (!isset($_SERVER['HTTP_REFERER']))
		return 'error.noref';
	// only case here is the "game" page, destination of the login script (which is allowed to be hosted offsite)
	elseif ($page == 'game')
		return 'main';
	// error trap - all pages beyond this point require a referer from within the site itself
	elseif (!stristr($_SERVER['HTTP_REFERER'], URL_BASE))
		return 'error.badref';
	// pages that only work in-game
	elseif (in_array($page, array(
			'main', 'validate', 'revalidate', 'banner',
			'admin/users', 'admin/empires', 'admin/clans', 'admin/market', 'admin/messages', 'admin/permissions', 'admin/log', 'admin/round', 'admin/history',
			'guide', 'status', 'scores', 'graveyard', 'search', 'news', 'contacts', 'clanstats',
			'farm', 'cash', 'land', 'build', 'demolish',
			'pvtmarketbuy', 'pvtmarketsell', 'pubmarketbuy', 'pubmarketsell', 'bank', 'lottery',
			'aid', 'clan', 'clanforum', 'military', 'magic',
			'manage/user', 'manage/empire', 'manage/clan',
			'messages', 'delete', 'logout',
			)))
		return $page;
	// error trap - "no such page"
	else	return 'error.badpage';
}

$db = db_open(DB_TYPE, DB_SOCK, DB_HOST, DB_PORT, DB_USER, DB_PASS, DB_NAME);
if (!$db)
	error_503('ERROR_TITLE', lang('ERROR_DATABASE_OFFLINE'));

set_error_handler('error_handler', E_WARNING | E_NOTICE | E_STRICT | E_RECOVERABLE_ERROR);

// load world variables
$world = new prom_vars();
if (!$world->load())
	error_500('ERROR_TITLE', 'Unable to read world variables!');
if (!$world->check())
	error_503('ERROR_TITLE', 'Game setup is not yet complete.');

// If we're configured for cronless turn updates, check them now
$turns = new prom_turns();
if (!TURNS_CRONTAB)
	$turns->doUpdate(FALSE);

// define constants based on round start/end times
if (CUR_TIME < $world->round_time_begin)
{	// pre-registration
	define('ROUND_SIGNUP', TRUE);
	define('ROUND_STARTED', FALSE);
	define('ROUND_CLOSING', FALSE);
	define('ROUND_FINISHED', FALSE);
	define('TXT_TIMENOTICE', lang('ROUND_WILL_BEGIN', gmdate(lang('ROUND_WILL_BEGIN_FORMAT'), $world->round_time_begin - CUR_TIME)));
}
elseif (CUR_TIME < $world->round_time_closing)
{	// normal gameplay
	define('ROUND_SIGNUP', TRUE);
	define('ROUND_STARTED', TRUE);
	define('ROUND_CLOSING', FALSE);
	define('ROUND_FINISHED', FALSE);
}
elseif (CUR_TIME < $world->round_time_end)
{	// final week (or so)
	define('ROUND_SIGNUP', FALSE);
	define('ROUND_STARTED', TRUE);
	define('ROUND_CLOSING', TRUE);
	define('ROUND_FINISHED', FALSE);
	define('TXT_TIMENOTICE', lang('ROUND_WILL_END', gmdate(lang('ROUND_WILL_END_FORMAT'), $world->round_time_end - CUR_TIME)));
}
else
{	// end of round
	define('ROUND_SIGNUP', FALSE);
	define('ROUND_STARTED', FALSE);
	define('ROUND_CLOSING', FALSE);
	define('ROUND_FINISHED', TRUE);
	define('TXT_TIMENOTICE', lang('ROUND_HAS_ENDED'));
}

if ($ban = check_banned_ip($_SERVER['REMOTE_ADDR']))
{
	$ban_message = lang('YOU_ARE_BANNED',
		gmdate(lang('COMMON_TIME_FORMAT'), $ban['p_createtime']),
		($ban['p_reason']) ? $ban['p_reason'] : lang('BANNED_NO_REASON'),
		($ban['p_expire'] == 0) ? lang('BANNED_PERMANENT') : lang('BANNED_EXPIRES', gmdate(lang('COMMON_TIME_FORMAT'), $ban['p_expire'])),
		MAIL_ADMIN);

	error_403('ERROR_TITLE_ACCESS', $ban_message);
}

// Special variables parsed by page_header()
// Page title ("Promisance - whatever")
$title = '';

// Set to combination of UFLAG_MOD/UFLAG_ADMIN to indicate USER privileges required to load page
$needpriv = 0;

// Add entries to this array to request entities to be loaded and locked.
$lock = array('emp1' => 0, 'emp2' => 0, 'user1' => 0, 'user2' => 0, 'clan1' => 0, 'clan2' => 0, 'world' => 0);
// Set to an entity number, or use -1 (with all but emp2) to determine the ID automatically from the loaded empire
// If loading an entity fails, its value will be reset to 0
// Setting 'emp1' has no effect - it exists for logging purposes and is automatically set to the current empire ID
// Setting 'user1' or 'world' to anything other than -1 has no effect - they only allow auto-detection.

$triedpage = getFormVar('location', 'login');
$page = validate_location($triedpage);
$action = getFormVar('action');

// if they tried entering a really really long action, truncate it and log a warning
if (strlen($action) > 64)
{
	$oldaction = $action;
	$action = substr($action, 0, 64);
	logmsg(E_USER_NOTICE, 'action overflowed: '. $oldaction);
}

$errchk = explode('.', $page);
if ($errchk[0] == 'error')
{
	$message = '<table><tr><th>'. lang('SECURITY_TITLE') .'</th></tr><tr><td>'. lang('SECURITY_DESC') .'<br />';
	if ($errchk[1] == 'badpage')
		$message .= lang('SECURITY_BADPAGE', htmlspecialchars($triedpage)) .'<br />';
	elseif ($errchk[1] == 'badref')
		$message .= lang('SECURITY_BADREF', htmlspecialchars($triedpage), htmlspecialchars($_SERVER['HTTP_REFERER']), URL_BASE) .'<br />';
	elseif ($errchk[1] == 'noref')
		$message .= lang('SECURITY_NOREF', htmlspecialchars($triedpage)) .'<br />';
	else	$message .= lang('SECURITY_UNKNOWN', htmlspecialchars($triedpage), $errchk[1]) .'<br />';
	$message .= lang('SECURITY_INSTRUCT', URL_BASE, MAIL_ADMIN) .'</td></tr></table>';

	logmsg(E_USER_ERROR, $triedpage);

	if ($errchk[1] == 'badpage')
		error_404('ERROR_TITLE', $message);
	else	error_403('ERROR_TITLE', $message);
}

require_once(PROM_BASEDIR .'pages/'. $page .'.php');
// Do not put anything after this line, since it may not be executed.
?>
