<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2011 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_5/pages/admin/locks.php $
 * $Id: locks.php 1848 2011-04-02 00:12:46Z Quietust $
 */

if (!defined('IN_GAME'))
	die('Access denied');

// This page is extremely stripped down, since it cannot write anything to the database
// and thus cannot establish a session - it must verify the username and password every time

function lockname ($id)
{
	if ($id == LOCK_SCRIPT)
		return lang('ADMIN_LOCKS_NAME_SCRIPT');
	if ($id == LOCK_HISTORY)
		return lang('ADMIN_LOCKS_NAME_HISTORY');
	if ($id == LOCK_RESET)
		return lang('ADMIN_LOCKS_NAME_RESET');
	if ($id == LOCK_NEW)
		return lang('ADMIN_LOCKS_NAME_NEW');
	if ($id == LOCK_TURNS)
		return lang('ADMIN_LOCKS_NAME_TURNS');
	return lang('ADMIN_LOCKS_NAME_USER', $id);
}

$access = 0;

// check if they're already logged in - if there are any problems, silently fall back to login form method
if (prom_session::check()) do
{
	prom_session::start();
	if (!prom_session::verify())
		break;

	$user1 = new prom_user($_SESSION['user']);
	if (!$user1->load())
		break;

	setlanguage($user1->u_lang);

	if (!($user1->u_flags & UFLAG_ADMIN))
		break;

	$access = 2;
} while (0);

if ($access == 0) do
{
	if (!isFormPost() || ($action == ''))
		break;

	$username = getFormVar('lock_username');
	$password = getFormVar('lock_password');

	if (empty($username))
	{
		notice(lang('INPUT_NEED_USERNAME'));
		break;
	}

	if (empty($password))
	{
		notice(lang('INPUT_NEED_PASSWORD'));
		break;
	}

	$user1 = new prom_user();
	if (!$user1->findName($username))
	{
		notice(lang('LOGIN_USER_NOT_FOUND'));
		break;
	}
	if (!$user1->load())
	{
		// this should be impossible if findName() succeeded
		notice(lang('LOGIN_USER_NOT_FOUND'));
		break;
	}
	if (!$user1->checkPassword($password))
	{
		notice(lang('INPUT_INCORRECT_PASSWORD'));
		break;
	}
	setlanguage($user1->u_lang);
	if (!($user1->u_flags & UFLAG_ADMIN))
	{
		notice(lang('ERROR_LOGIN_PAGE_PERMISSION'));
		break;
	}
	$access = 1;
} while (0);

if ($action == 'unlock') do
{
	if (!isFormPost())
		break;
	if ($access == 0)
	{
		notice(lang('ERROR_NOT_LOGGED_IN'));
		break;
	}
	$lock_u = getFormArr('lock_u');
	$lock_e = getFormArr('lock_e');
	$lock_c = getFormArr('lock_c');
	$lock_v = getFormArr('lock_v');

	$num = 0;
	foreach ($lock_u as $ent)
	{
		if ($db->unlockEntity(USER_TABLE, 'u_id', ENT_USER, fixInputNum($ent), 0))
			$num++;
		$num++;
	}
	if ($num > 0)
		notice(lang('ADMIN_LOCKS_USER_SUCCESS', $num));
	elseif (count($lock_u) > 0)
		notice(lang('ADMIN_LOCKS_USER_FAIL'));

	$num = 0;
	foreach ($lock_e as $ent)
	{
		if ($db->unlockEntity(EMPIRE_TABLE, 'e_id', ENT_EMPIRE, fixInputNum($ent), 0))
			$num++;
	}
	if ($num > 0)
		notice(lang('ADMIN_LOCKS_EMPIRE_SUCCESS', $num));
	elseif (count($lock_e) > 0)
		notice(lang('ADMIN_LOCKS_EMPIRE_FAIL'));

	$num = 0;
	foreach ($lock_c as $ent)
	{
		if ($db->unlockEntity(CLAN_TABLE, 'c_id', ENT_CLAN, fixInputNum($ent), 0))
			$num++;
	}
	if ($num > 0)
		notice(lang('ADMIN_LOCKS_CLAN_SUCCESS', $num));
	elseif (count($lock_c) > 0)
		notice(lang('ADMIN_LOCKS_CLAN_FAIL'));

	$num = 0;
	foreach ($lock_v as $ent)
	{
		if ($db->unlockEntity('DUAL', 'NULL', ENT_VARS, fixInputNum($ent), 0))
			$num++;
	}
	if ($num > 0)
		notice(lang('ADMIN_LOCKS_VARS_SUCCESS', $num));
	elseif (count($lock_v) > 0)
		notice(lang('ADMIN_LOCKS_VARS_FAIL'));
} while (0);

if ($access == 1)
	notice(lang('ADMIN_LOCKS_LOGIN_REMINDER'));

$html = new prom_html_compact();
$html->begin('ADMIN_LOCKS_TITLE');

notices();

if ($access)
{
?>
<form method="post" action="?location=admin/locks">
<div>
<input type="hidden" name="action" value="unlock" />
<?
	if ($access == 1)
	{
?>
<?=lang('LABEL_USERNAME')?> <input type="text" name="lock_username" size="8" /><br />
<?=lang('LABEL_PASSWORD')?> <input type="password" name="lock_password" size="8" /><br />
<?
	}
	$cnt = 0;

	// Users
	$result = $db->getLocks(USER_TABLE, 'u_id', ENT_USER);
	if (count($result))
	{
		$cnt++;
?>
<table border="1">
<tr><th><?=lang('ADMIN_LOCKS_COLUMN_USER')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKEDBY')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKDATE')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_UNLOCK')?></th></tr>
<?		foreach ($result as $ent)
		{ ?>
<tr><td><?=$ent['lock_id']?></td>
    <td><?=lockname($ent['lock_owner'])?></td>
    <td><?=$ent['lock_date']?></td>
    <td><?=checkbox('lock_u[]', '', $ent['lock_id'], FALSE, TRUE, 'lock_u_'. $ent['lock_id'])?></td></tr>
<?		} ?>
<tr><td colspan="4" class="ar"><a href="javascript:togglechecks('lock_u')"><?=lang('ADMIN_LOCKS_COLUMN_TOGGLE')?></a></td></tr>
</table>
<?	}

	// Empires
	$result = $db->getLocks(EMPIRE_TABLE, 'e_id', ENT_EMPIRE);
	if (count($result))
	{
		$cnt++;
?>
<table border="1">
<tr><th><?=lang('ADMIN_LOCKS_COLUMN_EMPIRE')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKEDBY')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKDATE')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_UNLOCK')?></th></tr>
<?		foreach ($result as $ent)
		{ ?>
<tr><td><?=$ent['lock_id']?></td>
    <td><?=lockname($ent['lock_owner'])?></td>
    <td><?=$ent['lock_date']?></td>
    <td><?=checkbox('lock_e[]', '', $ent['lock_id'], FALSE, TRUE, 'lock_e_'. $ent['lock_id'])?></td></tr>
<?		} ?>
<tr><td colspan="4" class="ar"><a href="javascript:togglechecks('lock_e')"><?=lang('ADMIN_LOCKS_COLUMN_TOGGLE')?></a></td></tr>
</table>
<?	}

	// Clans
	$result = $db->getLocks(CLAN_TABLE, 'c_id', ENT_CLAN);
	if (count($result))
	{
		$cnt++;
?>
<table border="1">
<tr><th><?=lang('ADMIN_LOCKS_COLUMN_CLAN')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKEDBY')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKDATE')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_UNLOCK')?></th></tr>
<?		foreach ($result as $ent)
		{ ?>
<tr><td><?=$ent['lock_id']?></td>
    <td><?=lockname($ent['lock_owner'])?></td>
    <td><?=$ent['lock_date']?></td>
    <td><?=checkbox('lock_c[]', '', $ent['lock_id'], FALSE, TRUE, 'lock_c_'. $ent['lock_id'])?></td></tr>
<?		} ?>
<tr><td colspan="4" class="ar"><a href="javascript:togglechecks('lock_c')"><?=lang('ADMIN_LOCKS_COLUMN_TOGGLE')?></a></td></tr>
</table>
<?	}

	// Vars
	$result = $db->getLocks('DUAL', 'NULL', ENT_VARS);
	if (count($result))
	{
		$cnt++;
?>
<table border="1">
<tr><th><?=lang('ADMIN_LOCKS_COLUMN_VARS')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKEDBY')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_LOCKDATE')?></th>
    <th><?=lang('ADMIN_LOCKS_COLUMN_UNLOCK')?></th></tr>
<?		foreach ($result as $ent)
		{ ?>
<tr><td><?=$ent['lock_id']?></td>
    <td><?=lockname($ent['lock_owner'])?></td>
    <td><?=$ent['lock_date']?></td>
    <td><?=checkbox('lock_v[]', '', $ent['lock_id'], FALSE, TRUE, 'lock_v_'. $ent['lock_id'])?></td></tr>
<?		} ?>
<tr><td colspan="4" class="ar"><a href="javascript:togglechecks('lock_v')"><?=lang('ADMIN_LOCKS_COLUMN_TOGGLE')?></a></td></tr>
</table>
<?	}
	if ($cnt == 0)
		echo lang('ADMIN_LOCKS_NONE_FOUND') .'<br />'."\n";
?>
<input type="submit" value="<?=lang(($cnt == 0) ? 'ADMIN_LOCKS_VIEW_SUBMIT' : 'ADMIN_LOCKS_UNLOCK_SUBMIT')?>" />
</div>
</form>
<?
}
else
{
?>
<form method="post" action="?location=admin/locks">
<div>
<input type="hidden" name="action" value="view" />
<?=lang('LABEL_USERNAME')?> <input type="text" name="lock_username" size="8" /><br />
<?=lang('LABEL_PASSWORD')?> <input type="password" name="lock_password" size="8" /><br />
<input type="submit" value="<?=lang('ADMIN_LOCKS_VIEW_SUBMIT')?>" />
</div>
</form>
<?
}
$html->end();
?>
